# Generated by Django 4.2.6 on 2023-11-23 08:27

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("df_survey", "0003_alter_question_options_remove_survey_questions_and_more"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="usersurvey",
            unique_together={("user", "survey")},
        ),
    ]
